/*
This file is part of LazyPC.
Written by Taylor Gurney, 2022-2024.
Public domain.
*/

#ifndef H_BITS_H
#define H_BITS_H

/* bit twiddling, and byte/word typedefs */

#define LB(n) ((byte)((n) & 0xff))
#define HB(n) ((byte)(((n) & 0xff00) >> 8))
#define LW(n) ((word)((n) & 0xffff))
#define HW(n) ((word)(((n) & 0xffff0000) >> 16))

#include <limits.h>

#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
#include <stdint.h>
typedef uint8_t byte;
typedef uint16_t word;
typedef uint32_t dword;
#else

#if UCHAR_MAX == 255
typedef unsigned char byte;
#elif CHAR_MAX == 255
typedef char byte;
#else
#error "Could not find a uint8"
#endif

#if UINT_MAX == 65535
typedef unsigned int word;
#elif USHRT_MAX == 65535
typedef unsigned short word;
#elif UCHAR_MAX == 65535
typedef unsigned char word;
#else
#error "Could not find a uint16"
#endif

#if UINT_MAX == 4294967295
typedef unsigned int dword;
#elif ULONG_MAX == 4294967295
typedef unsigned long dword;
#elif USHRT_MAX == 4294967295
typedef unsigned short dword;
#else
#error "Could not find a uint32"
#endif
#endif /* pre-c99 */

/* 8086 wraparound. No high memory area, no A20 line. 0xfffff is the max */
#define segoff_to_flat(seg, off) ((((seg) << 4) + (off)) & 0xfffff)
#define pack32(hi, lo) ((((dword)(hi)) << 16) | (lo))
#define sign16(w) ((w) & 0x8000)
#define sign8(b) ((b) & 0x80)
#define is_sign8(b) (!!((b) & 0x80))
#define is_sign16(w) (!!((w) & 0x8000))

#endif /* H_BITS_H */
