/*
This file is part of LazyPC.
Written by Taylor Gurney, 2022-2024.
Public domain.
*/

#include <stdio.h>

#include "bits.h"
#include "breakpoint.h"

word breakpoint_seg[BREAKPOINT_MAX];
word breakpoint_off[BREAKPOINT_MAX];
dword breakpoint_abs[BREAKPOINT_MAX];
byte breakpoints = 0;

int breakpoint_hit(word cs, word ip) {
    byte bit = 1;
    int i;
    dword addr = segoff_to_flat(cs, ip);
    for (i = 0; i < BREAKPOINT_MAX; i++) {
        if (breakpoints & bit) {
            if (breakpoint_abs[i] == addr) {
                return i;
            }
        }
        bit <<= 1;
    }
    return -1;
}

int breakpoint_set(word cs, word ip) {
    int i;
    byte bit = 1;
    for (i = 0; i < BREAKPOINT_MAX; i++) {
        if (!(breakpoints & bit)) {
            breakpoint_seg[i] = cs;
            breakpoint_off[i] = ip;
            breakpoint_abs[i] = segoff_to_flat(cs, ip);
            breakpoints |= bit;
            return i;
        }
        bit <<= 1;
    }
    return -1;
}

void breakpoint_clear(byte i) {
    byte mask = 1;
    while (i) {
        mask <<= 1;
        i--;
    }
    breakpoints &= ~mask;
}

void breakpoint_print(void) {
    int i;
    byte mask = 1;
    for (i = 0; i < BREAKPOINT_MAX; i++) {
        if (breakpoints & mask) {
            printf("breakpoint %i at %04hx:%04hx (%05x)\n",
                   i, breakpoint_seg[i], breakpoint_off[i], breakpoint_abs[i]);
        }
        mask <<= 1;
    }
}
