/*
This file is part of LazyPC.
Written by Taylor Gurney, 2022-2024.
Public domain.
*/

#include <stdio.h>
#include <stdlib.h>

#include "bits.h"
#include "dma.h"

/* Intel 8237 DMA controller */

static word reg[8];
/*
static byte status_reg;
static byte command_reg;
*/
static byte flipflop = 0;

static void flip(void) {
    flipflop = (~flipflop) & 1;
}

byte dma_read_reg(byte which) {
    byte result = 0;
    which = which & 0x7;
    if (flipflop) {
        result = (reg[which] & 0xff00) >> 8; /* msb */
    } else {
        result = (reg[which] & 0xff); /* lsb */
    }
    flip();
    return result;
}

void dma_write_reg(byte which, byte val) {
    which = which & 0x7;
    if (flipflop) {
        reg[which] &= 0x00ff;
        reg[which] |= val << 8;
    } else {
        reg[which] &= 0xff00;
        reg[which] |= val;
    }
    flip();
}

void dma_dump_state(void) {
    puts("+----------------------------------------------------------------------+");
    printf("| dma  [%c]   0:%04hx 1:%04hx 2:%04hx 3:%04hx ",
           (int)(flipflop ? '>' : '<'),
           reg[0], reg[1], reg[2], reg[3]);
    printf("4:%04hx 5:%04hx 6:%04hx 7:%04hx   |\n",
           reg[4], reg[5], reg[6], reg[7]);
}

void dma_init(void) {
    int i;
    for (i = 0; i < 8; i++) {
        reg[i] = 0;
    }
    flipflop = 0;
}
