/*
This file is part of LazyPC.
Written by Taylor Gurney, 2022-2024.
Public domain.
*/

#ifndef H_F600_H
#define H_F600_H

/*
This is a default ROM to be loaded at f600:0000 if the user does not
specify a ROM to put there.

Normally IBM Cassette BASIC should be installed there, but unlike BIOS,
BASIC is not necessary for the PC to operate, so a minimal stub is good
enough for most purposes.

Note that if you want to run the BASIC.COM or BASICA.COM programs that
are available in the earlier versions of IBM PC-DOS, you need a real
Cassette BASIC ROM.

This stub prompts the user to insert a system disk and then press a
key. It then calls int 19h to try booting again.

The last byte is a checksum byte, to prevent the 'F600 ROM' BIOS POST
error code.
*/
#define F600_DEFAULT_ROM_LEN 128
static byte f600_default_rom[F600_DEFAULT_ROM_LEN] = {
    0x8c,0xc8,0x8e,0xd8,0xbe,0x35,0x00,0xe8,
    0x1d,0x00,0xb0,0x41,0x00,0xd0,0x31,0xdb,
    0xb4,0x0e,0xcd,0x10,0xbe,0x67,0x00,0xe8,
    0x0d,0x00,0x31,0xc0,0xcd,0x16,0x31,0xc0,
    0xcd,0x19,0xea,0xf0,0xff,0x00,0xf0,0xeb,
    0x06,0x31,0xdb,0xb4,0x0e,0xcd,0x10,0xac,
    0x84,0xc0,0x75,0xf7,0xc3,0x4e,0x6f,0x20,
    0x62,0x6f,0x6f,0x74,0x61,0x62,0x6c,0x65,
    0x20,0x64,0x69,0x73,0x6b,0x2e,0x0d,0x0a,
    0x49,0x6e,0x73,0x65,0x72,0x74,0x20,0x73,
    0x79,0x73,0x74,0x65,0x6d,0x20,0x64,0x69,
    0x73,0x6b,0x20,0x69,0x6e,0x74,0x6f,0x20,
    0x64,0x72,0x69,0x76,0x65,0x20,0x00,0x2c,
    0x20,0x74,0x68,0x65,0x6e,0x20,0x70,0x72,
    0x65,0x73,0x73,0x20,0x61,0x6e,0x79,0x20,
    0x6b,0x65,0x79,0x2e,0x0d,0x0a,0x00,0xcd
};

/* The source code, in NASM syntax, follows. */

/*
bits 16
cpu 8086

main:
    mov ax, cs
    mov ds, ax
    mov si, message1
    call print_string
    mov al, 'A'
    add al, dl
    xor bx, bx
    mov ah, 0x0e
    int 0x10
    mov si, message2
    call print_string
    xor ax, ax ; keyboard get key
    int 0x16 ; keyboard io
    xor ax, ax
    int 0x19 ; bootstrap
    jmp 0xf000:0xfff0   ; unreachable

print_string:
    jmp .cond
    xor bx, bx
.next:
    mov ah, 0x0e ; write tty
    int 0x10
.cond:
    lodsb
    test al, al
    jnz .next
    ret

message1:
    db "No bootable disk.",0x0d,0x0a,"Insert system disk into drive ",0
message2:
    db ", then press any key.",0x0d,0x0a,0

db 0xcd  ; checksum byte

times 32768 - ($ - $$) db 0 ; padding
*/

#endif /* H_F600_H */
