/*
This file is part of LazyPC.
Written by Taylor Gurney, 2022-2024.
Public domain.
*/

#include <stdio.h>
#include <stdlib.h>

#include "pic.h"

/* Intel 8259 Programmable Interrupt Controller (PIC) chip */

byte (*irq_func[8])(void);

static byte mask = 0xff;

/*
first two writes to INTA01 port after power on are for init and do not
affect the mask so we account for these here.
*/
static byte init_state = 0;

void pic_init(byte (**irqfunc)(void)) {
    int i;
    init_state = 0;
    mask = 0xff;
    for (i = 0; i < 8; i++) {
        irq_func[i] = irqfunc[i];
    }
}

void pic_dump_state(void) {
    puts("+---------------------------------------+----+----+----+---------------+");
        printf("| irqs enabled: [%c] [%c] [%c] [%c] [%c] [%c] [%c] [%c]        | pic init: [%c] |\n",
               !(mask & 1) ? '0' : ' ',
               !(mask & 2) ? '1' : ' ',
               !(mask & 4) ? '2' : ' ',
               !(mask & 8) ? '3' : ' ',
               !(mask & 16) ? '4' : ' ',
               !(mask & 32) ? '5' : ' ',
               !(mask & 64) ? '6' : ' ',
               !(mask & 128) ? '7' : ' ',
               !!init_state ? 'x' : ' ');
}

void pic_set_mask(byte m) {
    if (init_state < 2) {
        init_state++;
        return;
    }
    mask = m;
}

byte pic_get_mask(void) {
    return mask;
}

byte pic_poll(void) {
    byte raised = 0;
    int i = 0;
    for (i = 0; i < 8; i++) {
        if (!(mask & (1 << i)) && irq_func[i]()) {
            raised |= (1 << i);
        }
    }
    return raised;
}
